#! /bin/sh

#01db_listSEED.sh performs database scan on year by year basis.
# input: "config.scv"
# output: SQLITE yearly database table "listSEED" 
# execution: bash 01db_listSEED.sh
# 
#Author: Jiri Kvapil (j.kvapil@ig.cas.cz) - initial version December2017
#Acknowledgments: This work was supported by project COST-action ES1401 "TIDES" / COST CZ LD15029 Ministry of Education, Youth and Sports, Czech Republic
#

# read config file
while read line
do
  col1=`echo $line | awk -F\; '{print $1}'`
  col2=`echo $line | awk -F\; '{print $2}'`
  if [[ $col1 == 'Format' ]];then
   format=$col2
  elif [[ $col1 == 'Data' ]];then
   fold=$col2
  elif [[ $col1 == 'Work' ]];then
   work=$col2
  elif [[ $col1 == 'Name' ]];then
   name=$col2
  elif [[ $col1 == 'Convention' ]];then
   conv=$col2
  fi
done < config.csv

if [[ -z $name ]]; then
 name=SCAN
fi
if [[ -z $work ]]; then
 work=`pwd`
fi
foldwork=$work/$name
if [[ -d $foldwork ]]; then
  rm -rf $foldwork
fi
mkdir $foldwork

echo SCAN: $fold
echo DATA FORMAT: $conv $format
echo OUT_DB: $foldwork
echo -e "\n"
# end config file

if [[ `echo $format $conv` == 'MSEED ANTELOPE' ]];then

# start scan data to database

for year in `ls -d ${fold}/[1-2]??? | awk -F/ '{print $NF}'`
do 
 echo $year

#read -p "Enter Year (YYYY): " year
 dbou="$foldwork"/"$name"_"$year".db
 tblou=listSEED
 sqlite3 "$dbou" "DROP TABLE IF EXISTS "$tblou""
 sqlite3 "$dbou" "CREATE TABLE "$tblou"
        (ID INTEGER PRIMARY KEY AUTOINCREMENT,
         FILE VARCHAR(64),
         YEAR VARCHAR(16),
         JDAY VARCHAR(16),
         PERMISSIONS VARCHAR(16),
         OWNER VARCHAR(16),
         SIZE INTEGER,
         DATETIME DATETIME)"

 for jday in `ls -d ${fold}/${year}/[0-3]?? | awk -F/ '{print $NF}'`
 do
  folder0=$fold/${year}/$jday
  for pathfile in `ls ${folder0}/* | grep -v '.tgz$'`
  do
   file=`echo $pathfile | awk -F"/" '{print $NF}'`
   permissions=`ls -l $pathfile | awk '{print $1}'`
   owner=`ls -l $pathfile | awk '{print $3}'`
   size=`ls -l $pathfile | awk '{print $5}'`
   daytime=`ls -l $pathfile | awk '{print $6,$7}'`

   sqlite3 "$dbou" "INSERT INTO "$tblou"
           (FILE,YEAR,JDAY,PERMISSIONS,OWNER,SIZE,DATETIME)
           VALUES('$file','$year','$jday','$permissions','$owner','$size','$daytime')"
  done
 done
done

else
 echo data format or convention NOT SUPPORTED
fi
