#! /bin/sh

#02db_checkNAME.sh performs file naming convention check on year by year basis, after year by year completion generates channel and station statistics
# input: database table "listSEED"
# output1: SQLITE yearly database tables "checkNAME_stations", "checkNAME_channels", "checkNAME_exceptions"
# output2: SQLITE auxiliary database tables "stations", "channels" 
# execution:bash 02db_checkNAME.sh
# 
#Author: Jiri Kvapil (j.kvapil@ig.cas.cz) - initial version December2017
#Acknowledgments: This work was supported by project COST-action ES1401 "TIDES" / COST CZ LD15029 Ministry of Education, Youth and Sports, Czech Republic
#

# read config file
while read line
do
  col1=`echo $line | awk -F\; '{print $1}'`
  col2=`echo $line | awk -F\; '{print $2}'`
  if [[ $col1 == 'Format' ]];then
   format=$col2
  elif [[ $col1 == 'Data' ]];then
   fold=$col2
  elif [[ $col1 == 'Work' ]];then
   work=$col2
  elif [[ $col1 == 'Name' ]];then
   name=$col2
  elif [[ $col1 == 'Convention' ]];then
   conv=$col2
  fi
done < config.csv

if [[ -z $name ]]; then
 name=SCAN
fi
if [[ -z $work ]]; then
 work=`pwd`
fi
foldwork=$work/$name

echo SCAN: $fold
echo DATA FORMAT: $conv $format
echo OUT_DB: $foldwork
echo -e "\n"
# end config file

if [[ `echo $format $conv` == 'MSEED ANTELOPE' ]];then

# start script

for YYYY in `ls $foldwork/"$name"_????.db | awk -F/ '{print $NF}' | awk -F_ '{print $2}' | awk -F. '{print $1}'`

do
 echo $YYYY
 dbin="$foldwork"/"$name"_"$YYYY".db
 tblin=listSEED
 tblou_exc=checkname_EXCEPTIONS
 tblou_chnl=checkname_CHANNELS
 tblou_sta=checkname_STATIONS

 tmpdir=`mktemp -d "$foldwork"/02tmp.XXX`
 trap 'rm -rf "$tmpdir"' EXIT

 tmp_l=`mktemp "$tmpdir"/tmp.XXX`
 tmp_d=`mktemp "$tmpdir"/tmp.XXX`
 tmp_t=`mktemp "$tmpdir"/tmp.XXX`
 tmp_s=`mktemp "$tmpdir"/tmp.XXX`
 tmp_lsta=`mktemp "$tmpdir"/tmp.XXX`
 tmp_lchnl=`mktemp "$tmpdir"/tmp.XXX`

 tmp_sqld=`mktemp "$tmpdir"/tmp.XXX`
 tmp_sqlf=`mktemp "$tmpdir"/tmp.XXX`
 sqlite3 -csv "$dbin" "select FILE,YEAR,JDAY from "$tblin"" > $tmp_sqld

 while read line
 do
  pathyear=`echo $line | awk -F, '{print $2}'`
  pathjday=`echo $line | awk -F, '{print $3}'`
  file=`echo $line | awk -F, '{print $1}'`

# wrap around proper name to fix bad naming in 2011/070
  if [[ "$file" == "BEN.CZ..BHZ.D.2011.070.0458" ]]; then file="BEN.BHZ.2011.070.04.58.00"; fi 
  if [[ "$file" == "BIT.CZ..BHZ.D.2011.070.0459" ]]; then file="BIT.BHZ.2011.070.04.59.00"; fi 
  if [[ "$file" == "LAZC.CZ..BHZ.D.2011.070.0459" ]]; then file="LAZC.BHZ.2011.070.04.59.00"; fi 
  if [[ "$file" == "PC26.CZ..BHZ.D.2011.070.0459" ]]; then file="PC26.BHZ.2011.070.04.59.00"; fi 
  if [[ "$file" == "VLC110311000000.BHZ" ]]; then file="VLC.BHZ.2011.070.00.00.15"; fi 
  sta=`echo $file | awk -F[.:] '{print $1}'`
  chnl=`echo $file | awk -F[.:] '{print $2}'`
  year=`echo $file | awk -F[.:] '{print $3}'`
  jday=`echo $file | awk -F[.:] '{print $4}'`
  hh=`echo $file | awk -F[.:] '{print $5}'`
  mm=`echo $file | awk -F[.:] '{print $6}'`
  ss=`echo $file | awk -F[.:] '{print substr($7,0,2)}'`
  app=`echo $file | awk -F[.:] '{print substr($7,3,9999)}'`
  suf=`echo $file | awk -F[.:] '{print $NF}'`
# end wrap around rename
  if [[ "$file" == "BEN.BHZ.2011.070.04.58.00" ]]; then file="BEN.CZ..BHZ.D.2011.070.0458"; fi 
  if [[ "$file" == "BIT.BHZ.2011.070.04.59.00" ]]; then file="BIT.CZ..BHZ.D.2011.070.0459"; fi 
  if [[ "$file" == "LAZC.BHZ.2011.070.04.59.00" ]]; then file="LAZC.CZ..BHZ.D.2011.070.0459"; fi 
  if [[ "$file" == "PC26.BHZ.2011.070.04.59.00" ]]; then file="PC26.CZ..BHZ.D.2011.070.045"; fi 
  if [[ "$file" == "VLC.BHZ.2011.070.00.00.15" ]]; then file="VLC110311000000.BHZ"; fi 


# check DATE
  if [[ "$pathyear""$pathjday" != "$year""$jday" ]] ||\
     [[ "$year" != [1-2][0-9][0-9][0-9] ]] ||\
     [[ "$jday" != [0-3][0-9][0-9] ]]; then
   echo $file,Bad DATE,"$pathyear"/"$pathjday"/"$file" >> $tmp_d
  fi

# check TIME
  if [[ $hh < 00 ]] || [[ $hh > 24 ]] ||\
     [[ $mm < 00 ]] || [[ $mm > 60 ]] ||\
     [[ $ss < 00 ]] || [[ $ss > 60 ]]; then
   echo $file,Bad TIME,"$pathyear"/"$pathday"/"$file" >> $tmp_t
  fi

# check SUFFIX othr than +
  if [[ -n $app ]] || [[ $ss != $suf ]]; then
   if [[ $app != '+' ]]; then
   echo $file,SUFFIX found,"$pathyear"/"$pathjday"/"$file" >> $tmp_s
   fi
  fi

# put list of stations and channels and any suffx to file before sorting
  echo "$sta","$chnl","$year""$jday" >> $tmp_l 
 done < $tmp_sqld

 if [[ ! -s $tmp_d ]]; then
 echo NONE,DATE OK,  >> $tmp_d; fi
 if [[ ! -s $tmp_t ]]; then
 echo NONE,TIME OK,  >> $tmp_t; fi

 aplus=`grep -c ',+$' $tmp_l`
 if [[ ! -s $tmp_s ]] || [[ $aplus -eq 0 ]]; then
 echo NONE,SUFFIX OK,  >> $tmp_s; fi
 if [[ $aplus > 0 ]]; then
 echo $file,'SUFFIX warning + found',"$pathyear"/"$pathjday"/"$file" >> $tmp_s; fi
# save exceptions to database
 echo 'FILE,CHECK,MESSAGE' > $tmp_sqlf
 cat $tmp_d $tmp_t $tmp_s >> $tmp_sqlf
 sqlite3 "$dbin" "DROP TABLE IF EXISTS tbltmp"
 sqlite3 -csv -header "$dbin" ".import "$tmp_sqlf" tbltmp "
 sqlite3 -header "$dbin" "DROP TABLE IF EXISTS "$tblou_exc";
          CREATE TABLE "$tblou_exc"
          (ID INTEGER PRIMARY KEY AUTOINCREMENT,
           FILE VARCHAR(64), 
           STATUS VARCHAR(32),
           INFO VARCHAR(64));
          INSERT INTO "$tblou_exc"(FILE,STATUS,INFO) 
           SELECT * FROM tbltmp;
          DROP TABLE tbltmp;"


#now work with lists of stations and channels
 awk -F, ' {if(Ymin[$1]>$3 || Ymin[$1]=="") {Ymin[$1]=$3}
            if(Ymax[$1]<$3 || Ymax[$1]=="") {Ymax[$1]=$3}}
           END{for (i in Ymin) {print i","Ymin[i]","Ymax[i]}}' $tmp_l |\
  awk -F, '{print $1","substr($2,0,4)","substr($2,5,3)","substr($3,0,4)","substr($3,5,3)}'>$tmp_lsta
# save stations to database                                                                                   
 echo 'STA,FDATE,LDATE,FJDATE,LJDATE' > $tmp_sqlf
 while read LINE
 do
  sta=`echo $LINE | awk -F, '{print $1}'`
  fyyyy=`echo $LINE | awk -F, '{print $2}'`
  fjday=`echo $LINE | awk -F, '{print $3}'`
  lyyyy=`echo $LINE | awk -F, '{print $4}'`
  ljday=`echo $LINE | awk -F, '{print $5}'`
  fdate=`date -d ""$fyyyy"/01/01 +"$fjday"DAY -1DAY" +%Y-%m-%d`
  ldate=`date -d ""$lyyyy"/01/01 +"$ljday"DAY -1DAY" +%Y-%m-%d`
 echo $sta,$fdate,$ldate,$fyyyy"-"$fjday,$lyyyy"-"$ljday >> $tmp_sqlf
 done < $tmp_lsta
 sqlite3 "$dbin" "DROP TABLE IF EXISTS tbltmp"
 sqlite3 -csv -header "$dbin" ".import "$tmp_sqlf" tbltmp "
 sqlite3 -header "$dbin" "DROP TABLE IF EXISTS "$tblou_sta";
          CREATE TABLE "$tblou_sta"
          (ID INTEGER PRIMARY KEY AUTOINCREMENT,
           STA VARCHAR(16),
           FDATE DATE,
           LDATE DATE,
           FJDATE DATE,
           LJDATE DATE);
          INSERT INTO "$tblou_sta"(STA,FDATE,LDATE,FJDATE,LJDATE)
           SELECT * FROM tbltmp;
          DROP TABLE tbltmp;"


 awk -F, ' {if(Ymin[$2]>$3 || Ymin[$2]=="") {Ymin[$2]=$3} 
            if(Ymax[$2]<$3 || Ymax[$2]=="") {Ymax[$2]=$3}}
           END{for (i in Ymin) {print i","Ymin[i]","Ymax[i]}}' $tmp_l |\
  awk -F, '{print $1","substr($2,0,4)","substr($2,5,3)","substr($3,0,4)","substr($3,5,3)}'>$tmp_lchnl
# save channels to database                                                                                     
 echo 'CHNL,FDATE,LDATE,FJDATE,LJDATE' > $tmp_sqlf
 while read LINE
 do
  chnl=`echo $LINE | awk -F, '{print $1}'`
  fyyyy=`echo $LINE | awk -F, '{print $2}'`
  fjday=`echo $LINE | awk -F, '{print $3}'`
  lyyyy=`echo $LINE | awk -F, '{print $4}'`
  ljday=`echo $LINE | awk -F, '{print $5}'`
  fdate=`date -d ""$fyyyy"/01/01 +"$fjday"DAY -1DAY" +%Y-%m-%d`
  ldate=`date -d ""$lyyyy"/01/01 +"$ljday"DAY -1DAY" +%Y-%m-%d`
 echo $chnl,$fdate,$ldate,$fyyyy"-"$fjday,$lyyyy"-"$ljday >> $tmp_sqlf
 done < $tmp_lchnl
 sqlite3 "$dbin" "DROP TABLE IF EXISTS tbltmp"
 sqlite3 -csv -header "$dbin" ".import "$tmp_sqlf" tbltmp "
 sqlite3 -header "$dbin" "DROP TABLE IF EXISTS "$tblou_chnl";                                                    
          CREATE TABLE "$tblou_chnl"                                                                             
          (ID INTEGER PRIMARY KEY AUTOINCREMENT,                                                                
           CHNL VARCHAR(16),                                                                                     
           FDATE DATE,                                                                                          
           LDATE DATE,                                                                                          
           FJDATE DATE,                                                                                         
           LJDATE DATE);                                                                                        
          INSERT INTO "$tblou_chnl"(CHNL,FDATE,LDATE,FJDATE,LJDATE)                                               
           SELECT * FROM tbltmp;                                                                                
          DROP TABLE tbltmp;"

 rm -rf $tmpdir
done
fi

# join station tables to print out list of unique statuins
# - emulate full outer join (keep unique from both) by twice left
# - attach and detach each year db and save to temp table to get around max attach db 10

dbou=$foldwork/"$name"_aux.db
tblou=stations

tmpdir=`mktemp -d $foldwork/04tmp.XXX`
trap 'rm -rf "$tmpdir"' EXIT

tmp=`mktemp "$tmpdir"/sql.XXX`
tmp_c=`mktemp "$tmpdir"/csv.XXX` 
tmp_o=`mktemp "$tmpdir"/out.XXX` 

i=1
for YYYY in `ls $foldwork/"$name"_????.db | awk -F/ '{print $NF}' | awk -F_ '{print $2}' | awk -F. '{print $1}'`
do
 dbin=$foldwork/"$name"_"$YYYY".db
 if [[ $i == 1 ]]; then
  echo attach database \'"$dbin"\' as db"$i" \; >> $tmp
 elif [[ $i == 2 ]] ; then
  echo attach database \'"$dbin"\' as db"$i" \; >> $tmp
  echo create temporary table XYZ"$i" as >> $tmp 
  echo  select sta\,fjdate\,ljdate from db1.checkname_STATIONS >> $tmp
  echo  left join db"$i".checkname_STATIONS using\(sta\,fjdate\,ljdate\) union >> $tmp 
  echo  select sta\,fjdate\,ljdate from db"$i".checkname_STATIONS >> $tmp
  echo  left join db1.checkname_STATIONS using\(sta\,fjdate\,ljdate\)\; >> $tmp 
  echo detach database db"$i" \; >> $tmp
 else
  echo attach database \'"$dbin"\' as db$i \; >> $tmp
  echo create temporary table XYZ$i as >> $tmp 
  echo  select sta\,fjdate\,ljdate from XYZ$((i-1)) >> $tmp
  echo  left join db$i.checkname_STATIONS using\(sta\,fjdate\,ljdate\) union >> $tmp 
  echo  select sta\,fjdate\,ljdate from db$i.checkname_STATIONS >> $tmp
  echo  left join XYZ$((i-1)) using\(sta\,fjdate\,ljdate\)\; >> $tmp 
  echo detach database db$i \; >> $tmp

 fi
 i=$((i+1)) 
done

echo select sta,min\(fjdate\) as minfjdate,max\(ljdate\) as maxljdate from XYZ$((i-1)) group by sta order by sta\; >> $tmp
sqlite3 -csv < $tmp > $tmp_c

echo sta,minfjdate,maxljdate,fnet,lnet,arch > $tmp_o
while read line
do
 sta=`echo $line | awk -F"," '{print $1}'`
 fyyyy=`echo $line | awk -F"," '{print $2}' | awk -F"-" '{print $1}'`
 fjday=`echo $line | awk -F"," '{print $2}' | awk -F"-" '{print $2}'`
 lyyyy=`echo $line | awk -F"," '{print $3}' | awk -F"-" '{print $1}'`
 ljday=`echo $line | awk -F"," '{print $3}' | awk -F"-" '{print $2}'`
 ffile=`find $fold/$fyyyy/$fjday -iname $sta.* | head -n 1`
 lfile=`find $fold/$lyyyy/$ljday -iname $sta.* | head -n 1`

 # extract network from mseed header using check_seed
 cs_list=`check_seed $ffile 2>/dev/null`
 fnet=`echo $cs_list | awk -F"Start of first record:" '{print $2}'|\
                         awk -F" " '{print $3}'`
# accomodate for missing channel
 if [[ $fnet == [0-9][0-9]/[0-9][0-9]/[0-9][0-9][0-9][0-9] ]]; then
  fnet=`echo $cs_list | awk -F"Start of first record:" '{print $2}'|\
                         awk -F" " '{print $2}'`
 fi
# REPEAT for lfile 
 cs_list=`check_seed $lfile 2>/dev/null`
 lnet=`echo $cs_list | awk -F"Start of first record:" '{print $2}'|\
                         awk -F" " '{print $3}'`
# accomodate for missing channel
 if [[ $lnet == [0-9][0-9]/[0-9][0-9]/[0-9][0-9][0-9][0-9] ]]; then
  lnet=`echo $cs_list | awk -F"Start of first record:" '{print $2}'|\
                         awk -F" " '{print $2}'`
 fi
 arch=NO
 if [[ $fnet == CZ || $lnet == CZ || $fnet == WB || $lnet == WB ]]; then arch=YES; fi
 echo $line,$fnet,$lnet,$arch >> $tmp_o
done < $tmp_c

sqlite3 -csv -header "$dbou" ".import "$tmp_o" tbltmp"
sqlite3 -header "$dbou" "DROP TABLE IF EXISTS "$tblou";
         CREATE TABLE "$tblou"
          (ID INTEGER PRIMARY KEY AUTOINCREMENT,STA VARCHAR(16),FJDATE VARCHAR(16),LJDATE VARCHAR(16),
              FNET VARCHAR(16),LNET VARCHAR(16),ARCH VARCHAR(16));
         INSERT INTO "$tblou"(STA,FJDATE,LJDATE,FNET,LNET,ARCH)
          SELECT STA,minfjdate,maxljdate,fnet,lnet,arch FROM tbltmp;
         DROP TABLE tbltmp;"
rm -rf $tmpdir

# join station tables to print out list of unique statuins
# - emulate full outer join (keep unique from both) by twice left
# - attach and detach each year db and save to temp table to get around max attach db 10

dbou=$foldwork/"$name"_aux.db
tblou=channels

tmpdir=`mktemp -d $foldwork/04tmp.XXX`
trap 'rm -rf "$tmpdir"' EXIT

tmp=`mktemp "$tmpdir"/sql.XXX`
tmp_c=`mktemp "$tmpdir"/csv.XXX` 
tmp_o=`mktemp "$tmpdir"/out.XXX` 

i=1
for YYYY in `ls $foldwork/"$name"_????.db | awk -F/ '{print $NF}' | awk -F_ '{print $2}' | awk -F. '{print $1}'`
do
 dbin=$foldwork/"$name"_"$YYYY".db
 if [[ $i == 1 ]]; then
  echo attach database \'"$dbin"\' as db"$i" \; >> $tmp
 elif [[ $i == 2 ]] ; then
  echo attach database \'"$dbin"\' as db"$i" \; >> $tmp
  echo create temporary table XYZ"$i" as >> $tmp 
  echo  select chnl\,fjdate\,ljdate from db1.checkname_CHANNELS >> $tmp
  echo  left join db"$i".checkname_CHANNELS using\(chnl\,fjdate\,ljdate\) union >> $tmp 
  echo  select chnl\,fjdate\,ljdate from db"$i".checkname_CHANNELS >> $tmp
  echo  left join db1.checkname_CHANNELS using\(chnl\,fjdate\,ljdate\)\; >> $tmp 
  echo detach database db"$i" \; >> $tmp
 else
  echo attach database \'"$dbin"\' as db$i \; >> $tmp
  echo create temporary table XYZ$i as >> $tmp 
  echo  select chnl\,fjdate\,ljdate from XYZ$((i-1)) >> $tmp
  echo  left join db$i.checkname_CHANNELS using\(chnl\,fjdate\,ljdate\) union >> $tmp 
  echo  select chnl\,fjdate\,ljdate from db$i.checkname_CHANNELS >> $tmp
  echo  left join XYZ$((i-1)) using\(chnl\,fjdate\,ljdate\)\; >> $tmp 
  echo detach database db$i \; >> $tmp

 fi
 i=$((i+1)) 
done

echo select chnl,min\(fjdate\) as minfjdate,max\(ljdate\) as maxljdate from XYZ$((i-1)) group by chnl order by chnl\; >> $tmp
sqlite3 -csv < $tmp > $tmp_c

echo chnl,minfjdate,maxljdate,arch > $tmp_o
while read line
do
 arch=YES
 echo $line,$arch >> $tmp_o
done < $tmp_c

sqlite3 -csv -header "$dbou" ".import "$tmp_o" tbltmp"
sqlite3 -header "$dbou" "DROP TABLE IF EXISTS "$tblou";
         CREATE TABLE "$tblou"
          (ID INTEGER PRIMARY KEY AUTOINCREMENT,CHNL VARCHAR(16),FJDATE VARCHAR(16),LJDATE VARCHAR(16),
              ARCH VARCHAR(16));
         INSERT INTO "$tblou"(CHNL,FJDATE,LJDATE,ARCH)
          SELECT CHNL,minfjdate,maxljdate,arch FROM tbltmp;
         DROP TABLE tbltmp;"
rm -rf $tmpdir

