
#03db_readbackOBSPY.sh performs data format check by readback with Obspy software.
# input: SQLITE table "listSEED"
# output: SQLITE yearly database table "checkseed" 
# execution: python 03db_readbackOBSPY.sh
# 
#Author: Jiri Kvapil (j.kvapil@ig.cas.cz) - initial version December2017
#Acknowledgments: This work was supported by project COST-action ES1401 "TIDES" / COST CZ LD15029 Ministry of Education, Youth and Sports, Czech Republic
#


import csv,re,glob,os,shutil,warnings,sqlite3,tempfile,struct,subprocess
from obspy import read

format=[]
fold=[]
workfold=[]
name=[]
conv=[]
with open("config.csv") as f:
 reader = csv.reader(f,delimiter=';')
 for row in reader:
  if row[0]=='Name':
    name=row[1]
  elif row[0]=='Format':
    format=row[1]
  elif row[0]=='Data':
    fold=row[1]
  elif row[0]=='Work':
    workfold=row[1]
  elif row[0]=='Convention':
    conv=row[1]

if name=='':
  name='SCAN'
if workfold=='':
  workfold=os.getcwd()

print(name)
print(format,conv)
print(fold)
print(workfold)


if str(format)+' '+str(conv) == 'MSEED ANTELOPE':
 print('OK')
else:
 print('data format or convention NOT SUPPORTED')

os.chdir(str(workfold)+'/'+str(name))
#create tmp folder  
tmpdir = tempfile.mkdtemp(prefix='07tmp.',dir=str(workfold)+'/'+str(name))

for file in glob.glob(str(name)+'*[1-2][0-9][0-9][0-9].db'):
 YYYY=str(file.split(".db")[0].split("_")[1])
 print(file,YYYY)
 dbin=str(workfold)+'/'+str(name)+'/'+file
 tblin='listSEED'
# tblou='obsPY' #OUTPUT NAME NEEDS TO BE CODE EXOLICITELY IN SQLITE (VARIABLES NOT ACCEPTED)

#prepare tmp file with headers
 predictable_filename='tmpfile_'+YYYY
 tmppath=os.path.join(tmpdir, predictable_filename )
 headerCSV = ("ID","FILE","STATUS","FORMAT","COUNT","ASAMPLING","NPTS",\
               "MAX","MEAN","STD","SIZE_CHECK","")
#write2tmp START
 with open(tmppath, "w") as tmp:
  wr=csv.writer(tmp)
  wr.writerows([headerCSV])
  tmp.close() # write2tmp END
     
# connect to sqlite database
 conn=sqlite3.connect(dbin)
 c=conn.cursor()

# go through sqlite table line by line
 for lines in c.execute('select ID,FILE,YEAR,JDAY,SIZE from listSEED'):
#  print(lines)
  slash = '/'
  pathSEED = fold + slash + lines[2] + slash + lines[3] + slash + lines[1] 
#  print(pathSEED)
  seedname = lines[1]   
  lineout=[lines[0],lines[1]]
  if "tgz" in seedname:
   flag="Error"
   append="BAD_FORMAT","TGZ","NA","NA","NA","NA","NA","NA","NA"
   lineout+=append
  elif lines[4] == 0:
   flag="Error"
   append="NO_DATA","NA","NA","NA","NA","NA","NA","NA","NA"
   lineout+=append         
  else:
   try:
    with warnings.catch_warnings():
     warnings.filterwarnings('error')
     try:
      st = read(pathSEED)
      format=st[0].stats.mseed.encoding 
      count=st.count() #stream trace count
      mean=0 #average trace mean amplitude per stream
      std=0 #stream std. dev. per stream
      max=0 #max amp per stream
      sampling=0 #average trace sampling per stream
      npts=0 #stream total samples
      count_nz=0 #none-zero point trace count
      for i,st in enumerate(st):
       if st.stats.npts > 0: #skip zero sample traces
        count_nz+=1
        sampling+=st.stats.sampling_rate
        npts+=st.stats.npts
        if st.stats.mseed.encoding != "ASCII":
         mean+=abs(st.data.mean())
         std+=abs(st.data.std())
         if abs(st.data.max()) > max:
          max=abs(st.data.max())
      sampling=sampling/count_nz
      mean=mean/count_nz
      std=std/count_nz
      flag="OK"
      lineout.append("OK")
      lineout.append(format)
      lineout.append(count)
      lineout.append('{0:.0f}'.format(sampling))
      lineout.append('{0:.0f}'.format(npts))
      if st.stats.mseed.encoding != "ASCII":  
       lineout.append('{0:.3f}'.format(max))
       lineout.append('{0:.3f}'.format(mean))
       lineout.append('{0:.3f}'.format(std))
      else:
       lineout.append("NA")
       lineout.append("NA")
       lineout.append("NA")
      lineout.append("NA")      
     except Warning:
      flag="Warning"
     except TypeError:
      flag="Error"
      append="TYPE_ERROR","NA","NA","NA","NA","NA","NA","NA","NA"
      lineout+=append
      pass
     except:
      isize=lines[4]
      tmpdir_cp=tmpdir + slash + 'work'+ str(lines[0])
      pathSEED_cp = tmpdir_cp + slash + lines[1]
      pathSEED_xf = tmpdir_cp + slash + lines[1] + '_xf'
#      print('EXTR_FILE START FOR ' + lines[1] + ' ' + str(lines[4]))
      subprocess.call('mkdir '+ tmpdir_cp,shell=True)  
      subprocess.call('cp '+ pathSEED +' '+pathSEED_cp,shell=True)    
      subprocess.call('extr_file '+ pathSEED_cp + ' -n o -o ' + tmpdir_cp + ' > /dev/null',shell=True)
      subprocess.call('rm -rf '+ pathSEED_cp,shell=True)
      extfSEED=os.listdir(tmpdir_cp)
      subprocess.call('mv '+ tmpdir_cp + slash + extfSEED[0] +' '+pathSEED_xf,shell=True)
      osize=os.path.getsize(pathSEED_xf)
      dsize=isize - osize
#      subprocess.call('ls -l '+ tmpdir_cp,shell=True)
#     print(str(isize) +','+ str(osize) +','+ str(dsize))
      try:
       st = read(pathSEED_xf)
       format=st[0].stats.mseed.encoding
       subprocess.call('rm -rf '+ tmpdir_cp,shell=True)
       count=st.count() #stream trace count
       mean=0 #average trace mean amplitude per stream
       std=0 #stream std. dev. per stream
       max=0 #max amp per stream
       sampling=0 #average trace sampling per stream
       npts=0 #stream total samples
       count_nz=0 #none-zero point trace count
       for i,st in enumerate(st):
        if st.stats.npts > 0: #skip zero sample traces
         count_nz+=1
         sampling+=st.stats.sampling_rate
         npts+=st.stats.npts
         if st.stats.mseed.encoding != "ASCII":
          mean+=abs(st.data.mean())
          std+=abs(st.data.std())
          if abs(st.data.max()) > max:
           max=abs(st.data.max())
       sampling=sampling/count_nz
       mean=mean/count_nz
       std=std/count_nz
       flag="OK_XFwrap"
       lineout.append("OK_EXTRFILE")
       lineout.append(format)
       lineout.append(count)
       lineout.append('{0:.0f}'.format(sampling))
       lineout.append('{0:.0f}'.format(npts))
       if st.stats.mseed.encoding != "ASCII":
        lineout.append('{0:.3f}'.format(max))
        lineout.append('{0:.3f}'.format(mean))
        lineout.append('{0:.3f}'.format(std))
       else:
        lineout.append("NA")
        lineout.append("NA")
        lineout.append("NA")                  
       lineout.append(dsize)
      except ValueError:
       flag="Error"
       append="VALUE_ERROR","NA","NA","NA","NA","NA","NA","NA","NA"
       lineout+=append
       subprocess.call('rm -rf '+ tmpdir_cp,shell=True)
       pass
      except struct.error:
       flag="Error"
       append="STRUCT_ERROR","NA","NA","NA","NA","NA","NA","NA","NA"
       lineout+=append
       subprocess.call('rm -rf '+ tmpdir_cp,shell=True)
       pass
      except:
       flag="Error"
       append="UNEXPECTED_ERROR","NA","NA","NA","NA","NA","NA","NA","NA"
       lineout+=append
       subprocess.call('rm -rf '+ tmpdir_cp,shell=True)           
       pass
   except IndexError:
    flag="Error"
    append="INDEX_ERROR","NA","NA","NA","NA","NA","NA","NA","NA"
    lineout+=append
    pass                 
# populate registers for warning
  if flag=="Warning":
   warnings.filterwarnings('ignore')
   st = read(pathSEED)
   format=st[0].stats.mseed.encoding
   count=st.count() #stream trace count
   mean=0 #average trace mean amplitude per stream
   std=0 #stream std. dev. per stream
   max=0 #max amp per stream
   sampling=0 #average trace sampling per stream
   npts=0 #stream total samples
   count_nz=0 #none-zero point trace count
   for i,st in enumerate(st):
    if st.stats.npts > 0: #skip zero sample traces
     count_nz+=1
     sampling+=st.stats.sampling_rate
     npts+=st.stats.npts
     if st.stats.mseed.encoding != "ASCII":
      mean+=abs(st.data.mean())
      std+=abs(st.data.std())
      if abs(st.data.max()) > max:
       max=abs(st.data.max())
   sampling=sampling/count_nz
   mean=mean/count_nz
   std=std/count_nz
   lineout.append("READ_WARNING")
   lineout.append(format)
   lineout.append(count)
   lineout.append('{0:.0f}'.format(sampling))
   lineout.append('{0:.0f}'.format(npts))
   if st.stats.mseed.encoding != "ASCII":      
    lineout.append('{0:.3f}'.format(max))
    lineout.append('{0:.3f}'.format(mean))
    lineout.append('{0:.3f}'.format(std))
   else:
    lineout.append("NA")
    lineout.append("NA")
    lineout.append("NA")
   lineout.append("NA")
#write2tmp START
#  print(lineout)
  with open(tmppath, "a") as tmp:
   wr=csv.writer(tmp)
   wr.writerows([lineout])
   tmp.close() # write2tmp END
#readback what is written there
# with open(tmppath, 'r') as fin:
#  print(fin.read())

#import tmp into sqlite
 c.execute('''DROP TABLE IF EXISTS obsPY''')
 conn.commit()
 c.execute('''CREATE TABLE obsPY (
            ID INTEGER,
            FILE VARCHAR(64),
            STATUS VARCHAR(32),
            FORMAT VARCHAR(16),
            COUNT INTEGER,
            ASAMPLING INTEGER,
            NPTS INTEGER,
            MAX INTEGER,
            MEAN INTEGER,
            STD INTEGER,
            SIZE_CHECK);''')                                                                                
 with open(tmppath,'r') as fin:
  dr=csv.DictReader(fin)
  to_db = [(i['ID'],i['FILE'],i['STATUS'],i['FORMAT'],i['COUNT'],i['ASAMPLING'],\
            i['NPTS'],i['MAX'],i['MEAN'],i['STD'],i['SIZE_CHECK']) for i in dr]
#  print(fin.read())                                                                                                                                                                                           
 c.executemany('''INSERT INTO obsPY (
                 ID,
                 FILE,
                 STATUS,
                 FORMAT,
                 COUNT,
                 ASAMPLING,
                 NPTS,
                 MAX,
                 MEAN,
                 STD,
                 SIZE_CHECK)
                 VALUES(?,?,?,?,?,?,?,?,?,?,?)''',to_db)
 conn.commit()
# get a rid of tmp data
 os.remove(tmppath)     
shutil.rmtree(tmpdir)
