#! /bin/sh

#03db_readbackSC.sh performs data format check by readback with SeisComp software (checkseed).
# input: SQLITE table "listSEED"
# output: SQLITE yearly database table "checkseed" 
# execution: bash 03db_readbackSC.sh
# 
#Author: Jiri Kvapil (j.kvapil@ig.cas.cz) - initial version December2017
#Acknowledgments: This work was supported by project COST-action ES1401 "TIDES" / COST CZ LD15029 Ministry of Education, Youth and Sports, Czech Republic
#

# read config file
while read line
do
  col1=`echo $line | awk -F\; '{print $1}'`
  col2=`echo $line | awk -F\; '{print $2}'`
  if [[ $col1 == 'Format' ]];then
   format=$col2
  elif [[ $col1 == 'Data' ]];then
   fold=$col2
  elif [[ $col1 == 'Work' ]];then
   work=$col2
  elif [[ $col1 == 'Name' ]];then
   name=$col2
  elif [[ $col1 == 'Convention' ]];then
   conv=$col2
  fi
done < config.csv

if [[ -z $name ]]; then
 name=SCAN
fi
if [[ -z $work ]]; then
 work=`pwd`
fi
foldwork=$work/$name

echo SCAN: $fold
echo DATA FORMAT: $conv $format
echo OUT_DB: $foldwork
echo -e "\n"
# end config file

if [[ `echo $format $conv` == 'MSEED ANTELOPE' ]];then

# start script

for YYYY in `ls $foldwork/"$name"_????.db | awk -F/ '{print $NF}' | awk -F_ '{print $2}' | awk -F. '{print $1}'`

do
 echo $YYYY
 dbin=$foldwork/"$name"_"$YYYY".db
 tblin=listSEED
 tblou=checkSEED

 tmpdir=`mktemp -d $foldwork/05tmp.XXX`
 trap 'rm -rf "$tmpdir"' EXIT

 tmp_sqld=`mktemp "$tmpdir"/tmp.XXX`
 tmp_sqlf=`mktemp "$tmpdir"/tmp.XXX`
 sqlite3 -csv "$dbin" "select ID,FILE,YEAR,JDAY from "$tblin"" > $tmp_sqld


 echo "ID,FILE,STATUS,XCHECK_STA,XCHECK_CHNL,XCHECK_DATE,XCHECK_JDATE,XCHECK_TIME,TIME_DIFF,\
NETWORK,RECORD_LENGTH,COUNT_GAPS" > $tmp_sqlf 

 while read line
 do
  pathyear=`echo $line | awk -F, '{print $3}'`
  pathjday=`echo $line | awk -F, '{print $4}'`
  file=`echo $line | awk -F, '{print $2}'`
  id=`echo $line | awk -F, '{print $1}'`

  nsta=`echo $file | awk -F[.:] '{print $1}'`
  nchnl=`echo $file | awk -F[.:] '{print $2}'`
  nyear=`echo $file | awk -F[.:] '{print $3}'`
  njday=`echo $file | awk -F[.:] '{print $4}'`
  nhh=`echo $file | awk -F[.:] '{print $5}'`
  nmm=`echo $file | awk -F[.:] '{print $6}'`
  nss=`echo $file | awk -F[.:] '{print substr($7,0,2)}'`
  ntime=$nhh:$nmm:$nss
  napp=`echo $file | awk -F[.:] '{print substr($7,3,9999)}'`
  nsuf=`echo $file | awk -F[.:] '{print $NF}'`
  ndate=`TZ=UTC date -d "$nyear/01/01 +"$njday"DAY -1DAY" +%Y/%m/%d`
  njdate=$nyear-$njday
  nepoch=`TZ=UTC date -d "$ndate $nhh":"$nmm":"$nss" +%s`

  cs_list=`check_seed $fold/$pathyear/$pathjday/$file 2>/dev/null`
#  echo $cs_list >> junk_$YYYY
  dbout_status=INIT
# flag STATUS: OK or TimeError or Corrupted mseed
  if `echo $cs_list | grep -q -a --text 'Format code'`; then
   if `echo $cs_list | grep -q -v -a --text 'Skip record'`; then
    if `echo $cs_list | grep -q -v -a --text 'DEC_FIXHEAD error: time error'`;then
     if `echo $cs_list | grep -q -v -a --text 'Frame count error'`; then
      dbout_status=OK
# flag Frame count error w' / w'out End of last record
     else
      if `echo $cs_list | grep -q -a --text 'End of last record'`; then
       dbout_status=MSEED_FRAMECOUNT_WARNING
      else
       dbout_status=MSEED_FRAMECOUNT_ERROR
      fi
     fi
    else
     dbout_status=MSEED_TIME_ERROR
    fi
   else
    dbout_status=MSEED_CORRUPTED
   fi
  fi
# flag STATUS: bad format
  if `echo $cs_list | grep -q -a --text 'Skip record'`; then
   if `echo $cs_list | grep -q -v -a --text 'Format code'`; then
    dbout_status=FILE_ERROR_FORMAT
   fi
  fi
# flag STATUS: zero size or missing file
  if `echo $cs_list | grep -q -v -a --text 'Opened file'`; then
   if [[ ! -f $fold/$pathyear/$pathjday/$file ]]; then
   dbout_status=FILE_LOST
   else
    if [[ ! -s $fold/$pathyear/$pathjday/$file ]]; then
     dbout_status=FILE_ERROR_ZEROSIZE
    else
     dbout_status=FILE_ERROR_UNKNOWN
    fi
   fi 
  fi
# for STATUS OK or MSEED_TIME_ERROR or MSEED_FRAME_ERROR
  if [[ $dbout_status == OK ]] || [[ $dbout_status == MSEED_TIME_ERROR ]] ||\
     [[ $dbout_status == MSEED_FRAMECOUNT_ERROR ]] || [[ $dbout_status == MSEED_FRAMECOUNT_WARNING ]]; then
# count GAPS
   dbout_gaps=`echo $cs_list | grep -o 'Time-gap in input file' | wc -l`
# flag HEADER2NAME
   hsta=`echo $cs_list | awk -F"Start of first record:" '{print $2}'|\
                         awk -F" " '{print $1}'`
   hchnl=`echo $cs_list | awk -F"Start of first record:" '{print $2}'|\
                         awk -F" " '{print $2}'`
   hnet=`echo $cs_list | awk -F"Start of first record:" '{print $2}'|\
                         awk -F" " '{print $3}'`
# accomodate for missing network
   if [[ $hnet == [0-9][0-9]/[0-9][0-9]/[0-9][0-9][0-9][0-9] ]]; then
    cs_net=NULL
    fdate=`echo $cs_list | awk -F"Start of first record:" '{print $2}'|\
                           awk -F" " '{print $3}'`
    fyear=`echo ${fdate:6:4}`
    fjday=`TZ=UTC date -d ${fdate:6:4}/${fdate:3:2}/${fdate:0:2} '+%j'`
    ftime=`echo $cs_list | awk -F"Start of first record:" '{print $2}'|\
                          awk -F" " '{print $4}'`
    fhh=`echo $cs_list | awk -F"Start of first record:" '{print $2}'|\
                         awk -F" " '{print substr($4,0,2)}'`
    fmm=`echo $cs_list | awk -F"Start of first record:" '{print $2}'|\
                         awk -F" " '{print substr($4,4,2)}'`
    fss=`echo $cs_list | awk -F"Start of first record:" '{print $2}'|\
                         awk -F" " '{print substr($4,7,2)}'`
   else
    cs_net=$hnet
    fdate=`echo $cs_list | awk -F"Start of first record:" '{print $2}'|\
                           awk -F" " '{print $4}'`
    fyear=`echo ${fdate:6:4}`
    fjday=`TZ=UTC date -d ${fdate:6:4}/${fdate:3:2}/${fdate:0:2} '+%j'`
    ftime=`echo $cs_list | awk -F"Start of first record:" '{print $2}'|\
                          awk -F" " '{print $5}'`
    fhh=`echo $cs_list | awk -F"Start of first record:" '{print $2}'|\
                         awk -F" " '{print substr($5,0,2)}'`
    fmm=`echo $cs_list | awk -F"Start of first record:" '{print $2}'|\
                         awk -F" " '{print substr($5,4,2)}'`
    fss=`echo $cs_list | awk -F"Start of first record:" '{print $2}'|\
                         awk -F" " '{print substr($5,7,2)}'`
   fi
   hdate=`TZ=UTC date -d "$fyear/01/01 +"$fjday"DAY -1DAY" +%Y/%m/%d`
   hjdate=$fyear-$fjday
   hepoch=`TZ=UTC date -d "$hdate $ftime" +%s`
   ftime_diff=`echo "($hepoch - $nepoch)" | bc`

   if [[ $dbout_status == MSEED_FRAMECOUNT_ERROR ]];then
    ldate=$fdate
    ltime=$ftime
   else
    ldate=`echo $cs_list | awk -F"End of last record:" '{print $2}'|\
                              awk -F" " '{print $1}'`
    ltime=`echo $cs_list | awk -F"End of last record:" '{print $2}'|\
                                 awk -F" " '{print $2}'`
   fi
   dbout_sta=OK;dbout_chnl=OK;dbout_date=OK;dbout_jdate=OK;dbout_time=OK;dbout_diff=0;dbout_net=OK;
   if [[ $nsta != $hsta ]];then dbout_sta="$nsta vs $hsta";fi
   if [[ $nchnl != $hchnl ]];then dbout_chnl="$nchnl vs $hchnl";fi
   if [[ $hnet != *CZ* ]];then dbout_net="$cs_net vs CZ";fi
   if [[ $nyear$njday != $fyear$fjday ]];then dbout_date="$ndate vs $hdate" \
                                              dbout_jdate="$nyear.$njday vs $fyear.$fjday" ;fi
   if [[ $nepoch != $hepoch ]];then dbout_time="$ntime vs ${ftime:0:8}"; dbout_diff="$ftime_diff";fi
# flag record length
   frec_s=`TZ=UTC date -d "${fdate:6:4}/${fdate:3:2}/${fdate:0:2} ${ftime:0:8}" +%s`
   lrec_s=`TZ=UTC date -d "${ldate:6:4}/${ldate:3:2}/${ldate:0:2} ${ltime:0:8}" +%s`
   date_min=`TZ=UTC date -d "${fdate:6:4}/${fdate:3:2}/${fdate:0:2} 00:00:00" +%s`
   date_minpls=`TZ=UTC date -d "${fdate:6:4}/${fdate:3:2}/${fdate:0:2} 00:03:00" +%s`
   date_max=`TZ=UTC date -d "${fdate:6:4}/${fdate:3:2}/${fdate:0:2} 00:00:00 +1day" +%s`
   date_maxpls=`TZ=UTC date -d "${fdate:6:4}/${fdate:3:2}/${fdate:0:2} 00:03:00 +1day" +%s`
   dbout_length=INIT
   if [[ $frec_s -ge $date_min && $lrec_s -le $date_maxpls ]]; then
    if [[ $frec_s -ge $date_min && $frec_s -le $date_minpls &&\
          $lrec_s -ge $date_max && $lrec_s -le $date_maxpls ]]; then
     dbout_length=FULL_DAY;else
     dbout_length=PART_DAY
    fi;
   else
    cs_exceed_ls=`TZ=UTC date -d "${ldate:6:4}/${ldate:3:2}/${ldate:0:2}" +%s`
    cs_exceed_fs=`TZ=UTC date -d "${fdate:6:4}/${fdate:3:2}/${fdate:0:2}" +%s`
    cs_exceed_d=`echo "($cs_exceed_ls - $cs_exceed_fs) / 24 / 3600 " | bc` 
    dbout_length=DAYOVERFLOW_"$cs_exceed_d"D
   fi
   if [[ $dbout_status == MSEED_FRAME_ERROR ]];then
    dbout_length=NA
   fi
  else
   dbout_gaps=NA;dbout_length=NA
   dbout_sta=NA;dbout_chnl=NA;dbout_date=NA;bout_jdate=NA;dbout_time=NA;dbout_diff=NA;dbout_net=NA;
   hsta=NA; hchnl=NA; fdate=NA; fyear=NA;fjday=NA; ftime=NA
  fi       

  echo $id,$file,$dbout_status,$dbout_sta,$dbout_chnl,$dbout_date,$dbout_jdate,$dbout_time,\
$dbout_diff,$dbout_net,$dbout_length,$dbout_gaps >> $tmp_sqlf
 done < $tmp_sqld
 sqlite3 -csv -header "$dbin" ".import "$tmp_sqlf" tbltmp"
 sqlite3 -header "$dbin" "DROP TABLE IF EXISTS "$tblou";
        CREATE TABLE "$tblou"                                                                                                    
         (ID INTEGER,                                                     
          FILE VARCHAR(64),                                                                         
          STATUS VARCHAR(32),                                                                         
          XCHECK_STA VARCHAR(32),                                                                         
          XCHECK_CHNL VARCHAR(32),                                                                  
          XCHECK_DATE VARCHAR(32),
          XCHECK_JDATE VARCHAR(32),
          XCHECK_TIME VARCHAR(32),
          TIME_DIFF INTEGER,
          NETWORK VARCHAR(32),
          RECORD_LENGTH VARCHAR(32),
          COUNT_GAPS INTEGER);                                                                             
         INSERT INTO "$tblou"(ID,FILE,STATUS,XCHECK_STA,XCHECK_CHNL,\
XCHECK_DATE,XCHECK_JDATE,XCHECK_TIME,TIME_DIFF,NETWORK,RECORD_LENGTH,COUNT_GAPS)                       
          SELECT * FROM tbltmp;
         DROP TABLE tbltmp;"
 rm -rf $tmpdir
done
fi
