#! /bin/sh

#03db_readbackSH.sh performs data format check by readback with SeismicHandler software (sfdilst).
# input: SQLITE table "listSEED"
# output: SQLITE yearly database table "listseed" 
# execution: bash 03db_readbackSH.sh
# 
#Author: Jiri Kvapil (j.kvapil@ig.cas.cz) - initial version December2017
#Acknowledgments: This work was supported by project COST-action ES1401 "TIDES" / COST CZ LD15029 Ministry of Education, Youth and Sports, Czech Republic
#

# read config file
while read line
do
  col1=`echo $line | awk -F\; '{print $1}'`
  col2=`echo $line | awk -F\; '{print $2}'`
  if [[ $col1 == 'Format' ]];then
   format=$col2
  elif [[ $col1 == 'Data' ]];then
   fold=$col2
  elif [[ $col1 == 'Work' ]];then
   work=$col2
  elif [[ $col1 == 'Name' ]];then
   name=$col2
  elif [[ $col1 == 'Convention' ]];then
   conv=$col2
  fi
done < config.csv

if [[ -z $name ]]; then
 name=SCAN
fi
if [[ -z $work ]]; then
 work=`pwd`
fi
foldwork=$work/$name

echo SCAN: $fold
echo DATA FORMAT: $conv $format
echo OUT_DB: $foldwork
echo -e "\n"
# end config file

if [[ `echo $format $conv` == 'MSEED ANTELOPE' ]];then

# start script

# type ST to make this script work
# setting sfdlist variables, etc..
read -p "Did you type ST (Y / N): " ST_ON
if [[ $ST_ON = [nN][oO] || $ST_ON = [nN] ]]; then
 echo "Type ST and try again"

elif [[ $ST_ON = [yY][eE][sS] || $ST_ON = [yY] ]]; then
 for YYYY in `ls $foldwork/"$name"_????.db | awk -F/ '{print $NF}' | awk -F_ '{print $2}' | awk -F. '{print $1}'`
                                                                                           
 do
  echo $YYYY
  dbin=$foldwork/"$name"_"$YYYY".db
  tblin=listSEED
  tblou=sfdLIST

  tmpdir=`mktemp -d $foldwork/06tmp.XXX`
  trap 'rm -rf "$tmpdir"' EXIT

  tmp_sqld=`mktemp "$tmpdir"/tmp.XXX`
  tmp_sqlf=`mktemp "$tmpdir"/tmp.XXX`
  sqlite3 -csv "$dbin" "select ID,FILE,YEAR,JDAY from "$tblin"" > $tmp_sqld

  tmp_1=`mktemp "$tmpdir"/tmp.XXX`
  tmp_2=`mktemp "$tmpdir"/tmp.XXX`

 echo "ID,FILE,STATUS" > $tmp_sqlf

  while read line
  do
   pathyear=`echo $line | awk -F, '{print $3}'`
   pathjday=`echo $line | awk -F, '{print $4}'`
   file=`echo $line | awk -F, '{print $2}'`
   id=`echo $line | awk -F, '{print $1}'`

   $SH_UTIL/sfdlist $file $fold/$pathyear/$pathjday $tmp_1 quiet >& $tmp_2
     
   if [ ! -s "$tmp_1" ] && [ ! -s "$tmp_2" ];then
    sfd_status=NO_DATA
   elif [ -s "$tmp_1" ] && [ ! -s "$tmp_2" ];then
    sfd_status=OK
   elif [ ! -s "$tmp_1" ] && [ -s "$tmp_2" ];then
    sfd_status=NOT_SEED
   else
    sfd_status=INIT
   fi
   echo $id,$file,$sfd_status >> $tmp_sqlf
  done < $tmp_sqld
  sqlite3 -csv -header "$dbin" ".import "$tmp_sqlf" tbltmp"
  sqlite3 -header "$dbin" "DROP TABLE IF EXISTS "$tblou";                                  
        CREATE TABLE "$tblou"
         (ID INTEGER,
          FILE VARCHAR(64),
          STATUS VARCHAR(32));
         INSERT INTO "$tblou"(ID,FILE,STATUS)
          SELECT * FROM tbltmp;                                                           
         DROP TABLE tbltmp;"
  rm -rf $tmpdir
 done
else
echo try again
fi
fi
